#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>

#ifdef USE_NLLIBC
#include <nllibc.h>
#endif

#include "lib.h"
#include "objlist.h"
#include "type.h"
#include "defines.h"
#include "syntax.h"
#include "lex.h"
#include "asm.h"
#include "argument.h"

static int fd = 0;

static int read_func(char *buffer, int size)
{
  return read(fd, buffer, size);
}

static int verbose = 0;
static int silent = 0;

static argument_t args[] = {
  { "-v", ARGUMENT_TYPE_FLAG_ON, &verbose },
  { "-s", ARGUMENT_TYPE_FLAG_ON, &silent },
  { NULL, ARGUMENT_TYPE_NONE, NULL }
};

int main(int argc, char *argv[])
{
  objlist_t list, defines_list, symbols;
  area_t static_area;
  syntax_t syntax;
  defines_t defines;

  argument_read(&argc, argv, args);

  if (argc > 1) {
    fd = open(argv[1], O_RDONLY);
  }

  list = lex_list_read(read_func);

  defines_list = objlist_create(NULL);
  defines = defines_create();
  objlist_insert_head(defines_list, &defines->entry);
  symbols = objlist_create(NULL);
  static_area = area_create(C_TYPE_STATIC);

  list = syntax_list_read_structure(list, NULL, NULL, defines_list,
				    symbols, NULL, static_area, static_area);
  if (list != NULL)
    ERREXITAT(0, 1);

  syntax = syntax_create(C_TYPE_BLOCK, 0, 0, NULL);
  syntax->type = C_TYPE_BLOCK;
  syntax->obj.block.defines = defines;
  syntax->obj.block.symbols = symbols;
  syntax->obj.block.syntaxes = objlist_create(NULL);

  syntax_list_read_control(&syntax->obj.block);

  if (verbose) {
    area_print(static_area, 0);
    printf("\n");

    syntax_print(syntax, 0);
  }

  if (!silent) {
    asm_symbol_list(static_area->symbols);
  }

  return 0;
}
